#include "ScannerScore/ScannerSUser.h"

using namespace std;

//////////////////////////////////////////////////////////
////// Function contents to be defined by the user ///////
//////////////////////////////////////////////////////////
// WARNING: Don't change the prototypes of the template //
// functions  in this file since they are used by the   //
//     core routines. Only change their contents!       //
//////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
////////////// Member Function of 2RxSM //////////////////
//////////////////////////////////////////////////////////
// These functions will only be called when you use a   //
// TwoHDM input file.                                   //
//////////////////////////////////////////////////////////

void TwoRxSM::UserInitCalcs(void){
  //////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR INITIAL CALCULATIONS BEFORE THE SCAN STARTS //
  //////////////////////////////////////////////////////////////////////////
  
  //// Setting up the output print options
  //// leave the 1st argument at 1 if you want print points 1 by 1
  //// The 2nd argument indicates how many variables you want to print per point
  double num_of_br=nHzero*16;
  setprint(1,22+num_of_br);
    
  //// Use this vector to print/keep any extra data you calculate on UserAnalysis
  //// Keep it at 0 if you have no use for it
  extra_data.resize(0);
  
#ifdef HDECAYMODE
#ifdef HBMODE
  //// Initialising HiggsBounds!
  HiggsBoundsAnalysis.InitialiseHiggsBounds();
#ifdef HSMODE  
  //// Initialising HiggsSignals!
  //int Np=0; //Number of free parameters of the model
  initialize_higgssignals_latestresults_(nHzero,nHplus);
  //setup_nparam_(Np); // Set number of parameters for the model (no call defaults to 0)
  int pdf=2; //choose probability density function of Higgs mass arround the measured central value (HiggsSignals)
  setup_pdf_(pdf);
#endif
#endif
#endif
}

bool TwoRxSM::UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing){
  //////////////////////////////////////////////////////
  // ENTER CODE FOR YOUR TESTS/OUTPUT DURING THE SCAN //
  //////////////////////////////////////////////////////
  
  //--------------------------------------------------//
  
  //Test Electroweak precision observables
  if(not TestSTU_2sigma_xSM2_mixall(Mass,Mixing))
    return false;
  
  compute_scalar_widths();

#ifdef HBMODE
//Current implementation uses Hdecay to compute branching ratios for HiggsBounds. Alternatively you can set the values of HiggsBoundsAnalysis using your own calculations.
#ifdef HDECAYMODE
  if(not PrepareHiggsBoundsInput())
    return false;
  int HBresult,chan,ncombined;
  double obsratio;
  HiggsBoundsAnalysis.run(HBresult,chan,obsratio,ncombined);
#ifdef HSMODE  
  //Call to run Higgs Signals 
  int mode=1; //input
  //output variables
  double csqmu;
  double csqmh;
  double csqtot;
  int  nobs;
  run_higgssignals_(mode,csqmu,csqmh,csqtot,nobs,Pvalue);

  if(HBresult==0 || Pvalue<0.0027)
    return false; //Exclude point if excluded from non-observation in the HB search channels or if probability of model (calculated with higgs signals) given the observed signal rates is too low (3sigma). Note that 2sigma is more standard, however, we can recover the 1 and 2 sigma regions by post-processing using the provided pvalue on output.
#endif  
#endif
#endif

#ifdef MICROMEGASMODE
//if micromegas is on AND we have DM candidate 
  //std::clog<<"Inside micromegas compiler flag useranalysis"<<std::endl;
  if(DM_candidate){
  
    assignMOvalues();
  
    // Compute dark matter contributions from dark matter particle 1 (lightest)
    double Xf,cut=0.01,Beps=1e-5;
    
    // First the relic density contribution
    Omega=darkOmega(&Xf,1,Beps);

    //std::clog<<"Omega is "<< Omega<<" SigmaSI is "<<sigmaSI3<<"sigmaup is "<<sigmaup<<"mass is "<<Cp.Mass[2]<<endl;
    if(Omega>0.13 || Omega<0.11)
      return false;

    // spin_amplitudes: 0: pA0; 1: pAspin; 2: nA0; 3:nAspin
    double spin_amplitudes[4][2];	
    
    
    if(not test_nucleon_amplitudes(Omega,sigmaSI3,Cp.Mass[2],spin_amplitudes,sigmaup)){
      return false;
    }

    //std::clog<<"Omega is "<< Omega<<" SigmaSI is "<<sigmaSI3<<"sigmaup is "<<sigmaup<<"mass is "<<Cp.Mass[2]<<endl;
  }
#endif

  //MadGraph_Input();

  /*/--------------------------------------------------//
  extra_data[0]=Gamma011;
  extra_data[1]=Gamma012;
  extra_data[2]=Gamma022;
  extra_data[3]=Gamma100;
  extra_data[4]=Gamma102;
  extra_data[5]=Gamma122;
  extra_data[6]=Gamma200;
  extra_data[7]=Gamma201;
  extra_data[8]=Gamma211;
  //--------------------------------------------------/*/
  return true;
}

void TwoRxSM::what2print(){
//////////////////////////////////////////////////////////////////////////
//       CHOSE WHAT YOU WANT TO BE PRINTED TO THE OUTPUT FILE HERE      //
////////////////////////////////////////////////////////////////////////// 
// YOU NEED TO USE setprint() IN UserInitCalcs() TO DEFINE HOW MUCH     //
// DATA YOU WANT TO PRINT PER POINT (AND HOW MANY POINTS TO STORE       //
// BEFORE PRINTING).                                                    //
////////////////////////////////////////////////////////////////////////// 
  
  double data=0;
  //masses
  for(size_t i=0;i!=3;++i){
    points2print[store_vector_index][data]=Cp.Mass[i];
    ++data;
  }
  //Phi's
  points2print[store_vector_index][data]=Phi[2];
  ++data;
    for(size_t i=4;i!=6;++i){
    points2print[store_vector_index][data]=Phi[i];
    ++data;
  }
  //Couplings
    for(size_t i=0;i!=12;++i){
    points2print[store_vector_index][data]=Cp.L[i];
    ++data;
  }
  //Mixings
    for(size_t i=0;i!=3;++i){
    points2print[store_vector_index][data]=Mmixing[i][2];
    ++data;
  }
  //HiggsSignals' Pvalue
#ifdef HSMODE
  points2print[store_vector_index][data]=Pvalue;
  ++data;
#endif 
  
#ifdef HDECAYMODE
#ifdef HBMODE
    for(int i=0;i!=nHzero;++i){
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjss[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjcc[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjbb[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjmumu[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjtautau[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjWW[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjZZ[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjZga[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjgaga[i];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjgg[i];
    ++data;
    points2print[store_vector_index][data]=BRfrac*HdecayHsm["BR(Hsm -> t tbar)"];;
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjinvisible[i];
    ++data;
    for(int j=0;j!=3;++j){
      if(i!=j){
	points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjhihi[i][j];
	++data;
      }
    }
    points2print[store_vector_index][data]=Gamma012/HiggsBoundsAnalysis.MhGammaTot[i];;
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.MhGammaTot[i];
    ++data; 
  }
#endif
#endif
  
  //extra_data
  for(size_t i=0;i!=extra_data.size();++i){
    points2print[store_vector_index][data]=extra_data[i];
    ++data;
  }
} 

void TwoRxSM::UserFinalCalcs(void){
  ////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR FINAL CALCULATIONS AFTER THE SCAN IS DONE //
  ////////////////////////////////////////////////////////////////////////
  // Function for User defined final calculations to be executed after	//
  // the scan is finished						//
  ////////////////////////////////////////////////////////////////////////
    
}

bool TwoRxSM::CheckStability(LambdaRef & L)const{
  ///////////////////////////////////////////////////////////////////
  ///// TwoRxSM Potential Stability function                       /////
  ///////////////////////////////////////////////////////////////////
  ///// THIS FUNCTION IS ON A USER.CPP FILE MAINLY TO SHOW THE	/////
  ///// USER WHAT THE SCAN IS CALCULATING. WE WOULD ADVISE YOU	/////
  ///// NOT TO ALTER THE CONTENTS OF THIS FUNCTION.		/////
  ///////////////////////////////////////////////////////////////////

  // se algum dos quadraticos for negativo, return false
  if(L[9]<=0 || L[10]<=0 || L[11]<=0){
    return false;
  }
  
  // se todos os outros forem positivos, return true
  if(L[6]>0 && L[7]>0 && L[8]>0){
    return true;
  }
  
  // matriz de input e output
  double Min[][3]={
    {L[11] , L[6]/2. , L[8]/2. },
    {L[6]/2., L[10]/6., L[7]/2. },
    {L[8]/2., L[7]/2. , L[9]/6. }
  };

  double Mout[4][3];
  
  EigenSystemRsym(3, &Min[0][0],&Mout[0][0]);
  
  // verificação dos eigenvalues
  // se algum deles for falso, return false
  if(Mout[0][0]<0 || Mout[0][1]<0 || Mout[0][2]<0){
    return false;
  }
  
  // se conseguir passar por todos os teste, return true.
  return true;
}

bool TwoRxSM::CheckGlobal(PhiRef & Phi0,LambdaRef & L0,Potential & V)const{
  ///////////////////////////////////////////////////////////////////
  ///// TwoRxSM Global Minimum function                            /////
  ///////////////////////////////////////////////////////////////////
  ///// THIS FUNCTION IS ON A USER.CPP FILE MAINLY TO SHOW THE	/////
  ///// USER WHAT THE SCAN IS CALCULATING. WE WOULD ADVISE YOU	/////
  ///// NOT TO ALTER THE CONTENTS OF THIS FUNCTION.		/////
  ///////////////////////////////////////////////////////////////////
  
  //Bind references to auxiliary potential Vaux
  PhiRef Phi(V); 
  LambdaRef L(V);
  //Copy values from current parameter space point
  Phi.copy(Phi0);
  L.copy(L0);
  
  //// Compute potential at the current minimum
  double V0=V();
  if(V0>0){//Compare with stationary point at the origin which always has V=0 (in the SM case this is the tachyonic point)
    //std::clog<<"reject0"<<endl;
    return false;
  }
  double Vtest;
  double * atemp;
  atemp = new double [3+1];
  double * ztemp;
  ztemp = new double [2*3];
  //Possible minimum with v=0 and va=0
  //Expressions straight from mathematica!
  Phi[2]=0e0;
  Phi[4]=(-3*L[3] - pow(-24*L[1]*L[10] + 9*pow(L[3],2),0.5))/2./L[10];
  Phi[5]=0e0;
  double descrim=-8*L[1]*L[10] + 3*pow(L[3],2);
  Vtest=V();
  if(Vtest<V0 && descrim>0){
    //std::clog<<"reject1"<<endl;
    return false;
  }
  //Test case with flipped square root
  Phi[4]=(-3*L[3] + pow(-24*L[1]*L[10] + 9*pow(L[3],2),0.5))/2./L[10];
  
  Vtest=V();
  if(Vtest<V0 && descrim>0){
    //std::clog<<"reject1.1"<<endl;
    return false;
  }

  Phi.copy(Phi0);

  //Case with v=Phi[2]=0
  // Solution for vs, i.e. Phi[4]
  //Preparing coefficients for the cubic equation...
  atemp[0]=-(3*L[2]*L[5]);
  atemp[1]=(-3*L[5]*L[5] - 3*L[2]*L[7] + L[1]*L[9]);
  atemp[2]=(-9*L[5]*L[7] + L[3]*L[9])/2.;
  atemp[3]=(-9*L[7]*L[7] + L[9]*L[10])/6.;
  //Find all solutions of cubic equation...
  polyroots(atemp,ztemp,3);
  for(int i=0;i<3;++i){
    if(abs(ztemp[2*i+1])<1e-8*abs(ztemp[2*i])){
      Phi[2]=0e0;
      Phi[4]=ztemp[2*i];
      
      // Solution only exists if the modsquare of Phi[5] is positive (otherwise proceed to next case)
      double phi5square=3*(-2*L[2] - 2*L[5]*Phi[4] - L[7]*Phi[4]*Phi[4])/L[9];
      if(phi5square>0){
	Phi[5]=sqrt(phi5square);   
	
	//Bloco de testes aos Phi, utilizando as derivadas no ponto
	//Para estarem de acordo, têm que a=0, b=0 e c=0
	/*
	  double a=(1./2.)*L[0]*Phi[2] + (1./4.)*L[11]*Phi[2]*Phi[2]*Phi[2] + (1./2.)*L[4]*Phi[2]*Phi[4] + (1./4.)*L[6]*Phi[2]*Phi[4]*Phi[4] + (1./4.)*L[8]*Phi[2]*Phi[5]*Phi[5];
	  double b=(1./4.)*L[4]*Phi[2]*Phi[2] + L[1]*Phi[4] + (1./4.)*L[6]*Phi[2]*Phi[2]*Phi[4] + (1./2.)*L[3]*Phi[4]*Phi[4] + (1./6.)*L[10]*Phi[4]*Phi[4]*Phi[4] + (1./2.)*L[5]*Phi[5]*Phi[5] + (1./2.)*L[7]*Phi[4]*Phi[5]*Phi[5];
	  double c=L[2]*Phi[5] + (1./4.)*L[8]*Phi[2]*Phi[2]*Phi[5] + L[5]*Phi[4]*Phi[5] + (1./2.)*L[7]*Phi[4]*Phi[4]*Phi[5] + (1./6.)*L[9]*Phi[5]*Phi[5]*Phi[5];
	*/
	
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject2"<<endl;
	  return false;
	}
	
	// Test  Phi[5] negative case by flipping
	Phi[5]=-Phi[5]; 
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject3"<<endl;
	  return false;
	}
      }
    }
    Phi.copy(Phi0);   
    //clog<<"END"<<endl;
  }

  //Possible minimum with va=Phi[5]=0
  //Search solution for vs, i.e. Phi[4]
  atemp[0]=-L[0]*L[4]/2.;
  atemp[1]=(-L[4]*L[4]-L[0]*L[6]+2*L[1]*L[11])/2.;
  atemp[2]=(-3*L[4]*L[6]+2*L[3]*L[11])/4.;
  atemp[3]=(-3*L[6]*L[6]+2*L[10]*L[11])/12.;
  polyroots(atemp,ztemp,3);
  for(int i=0;i<3;++i){
    if(abs(ztemp[2*i+1])<1e-8*abs(ztemp[2*i])){
      Phi[5]=0e0;
      Phi[4]=ztemp[2*i];
      
      // Solution only exists if the modsquare of Phi[2] is positive (otherwise proceed to next case)
      double phi2square=(-2*L[0]-2*L[4]*Phi[4]-L[6]*Phi[4]*Phi[4])/L[11];
      if(phi2square>0){
	Phi[2]=sqrt(phi2square);            
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject4"<<endl;
	  return false;
	}
	
	// Test  Phi[2] negative case by flipping
	Phi[2]=-Phi[2]; 
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject5"<<endl;
	  return false;
	}
      }
      
    }
    Phi.copy(Phi0);   
    //clog<<"FIM"<<endl;
  }
  
  //Possible minimum with all values nonzero
  //Search solution for vs, i.e. Phi[4]
  atemp[0]=-3*L[2]*L[4]*L[8] - 3*L[0]*L[5]*L[8] + L[0]*L[4]*L[9] + 6*L[2]*L[5]*L[11];
  atemp[1]=-3*L[2]*L[6]*L[8] - 3*L[0]*L[7]*L[8] + L[0]*L[6]*L[9] + 6*L[2]*L[7]*L[11] - 6*L[4]*L[5]*L[8] + 3*L[1]*L[8]*L[8] + L[4]*L[4]*L[9] + 6*L[5]*L[5]*L[11] - 2*L[1]*L[9]*L[11];
  atemp[2]=(-9*L[4]*L[7]*L[8] + 3*L[3]*L[8]*L[8] + 3*L[4]*L[6]*L[9] - 2*L[3]*L[9]*L[11] - 9*L[5]*(L[6]*L[8] - 2*L[7]*L[11]))/2.;
  atemp[3]=(-18*L[6]*L[7]*L[8] + 3*L[6]*L[6]*L[9] + 3*L[8]*L[8]*L[10] + 2*(9*L[7]*L[7] - L[9]*L[10])*L[11])/6.;
  polyroots(atemp,ztemp,3);
  for(int i=0;i<3;++i){
    if(abs(ztemp[2*i+1])<1e-8*abs(ztemp[2*i])){
      Phi[4]=ztemp[2*i];
      
      //Testing again both modsquares (both must be positive)
      double phi5square=(3*(-2*L[0]*L[8] + 4*L[2]*L[11] - 2*L[4]*L[8]*Phi[4] + 4*L[5]*L[11]*Phi[4] - L[6]*L[8]*Phi[4]*Phi[4] + 2*L[7]*L[11]*Phi[4]*Phi[4]))/(3*L[8]*L[8] - 2*L[9]*L[11]);
      double phi2square=(-2*L[0]-2*L[4]*Phi[4]-L[6]*Phi[4]*Phi[4]-L[8]*Phi[5]*Phi[5])/L[11];

      if(phi5square>0 && phi2square>0 ){

	Phi[5]=sqrt(phi5square);
	Phi[2]=sqrt(phi2square);
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject6"<<endl;
	  return false;
	}
      
	// testing all remaining cases according to sign flips
	// Phi5 positive
	// Phi2 negative
	Phi[2]=-Phi[2];
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject7"<<endl;
	  return false;
	}

	// Phi5 negative
	// Phi2 positive
	Phi[5]=-Phi[5];
	Phi[2]=-Phi[2];
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject8"<<endl;
	  return false;
	}
	
	// Phi5 negative
	// Phi2 negative
	Phi[2]=-Phi[2];
	Vtest=V();
	if(Vtest<V0){
	  //std::clog<<"reject9"<<endl;
	  return false;
	}
      }
    }
    Phi.copy(Phi0);   
  }
  
  delete [] atemp;
  delete [] ztemp;
  
  return true;
}

//////////////////////////////////////////////////////////
//////// TwoRxSM default parameterizations         //////////
//////////////////////////////////////////////////////////
// These functions will be called when you use the TwoRxSM //
// class. TwoRxSM.potential does not have parametrizations //
// on by default, so they will not be called.		//
//////////////////////////////////////////////////////////

void TwoRxSM::MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE TwoRxSM.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////
  
}

void TwoRxSM::MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,vector< vector<double> > & MixInternal){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE TwoRxSM.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////

}

void TwoRxSM::MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const vector< vector<double> > & Mixing,const MixingparamVec & MixPar,vector<double> & L,vector<double> & Mass, MLparamVec & MLPar, vector<double> & IndependentMs,vector<double> & IndependentLs){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE TwoRxSM.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////

}
