#include "ScannerScore/ScannerSUser.h"

using namespace std;

//////////////////////////////////////////////////////////
////// Function contents to be defined by the user ///////
//////////////////////////////////////////////////////////
// WARNING: Don't change the prototypes of the template //
// functions  in this file since they are used by the   //
//     core routines. Only change their contents!       //
//////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
////////////// Member Function of xSM ////////////////////
//////////////////////////////////////////////////////////
// These functions will only be called when you use a   //
// TwoHDM input file.                                   //
//////////////////////////////////////////////////////////

//ofstream shdecay_out,hdecay_rescale_out;

void xSM2::UserInitCalcs(void){
  //////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR INITIAL CALCULATIONS BEFORE THE SCAN STARTS //
  //////////////////////////////////////////////////////////////////////////
  //// Setting up the output print options
  //// leave the 1st argument at 1 if you want print points 1 by 1
  
    //// Setting up the output print options
  //// leave the 1st argument at 1 if you want print points 1 by 1
  setprint(50,76);
  
  //// Use this vector to print/keep any extra data you calculate on UserAnalysis
  //// Keep it at 0 if you have no use for it
  extra_data.resize(0);
 
  Load_SM_noEW_SusHiPP_7to14();
 
#ifdef HBMODE
  //// Initialising HiggsBounds!
  HiggsBoundsAnalysis.InitialiseHiggsBounds();
#ifdef HSMODE  
  //// Initialising HiggsSignals!
  //int Np=0; //Number of free parameters of the model
  initialize_higgssignals_latestresults_(nHzero,nHplus);
  //setup_nparam_(Np); // Set number of parameters for the model (no call defaults to 0)
  int pdf=2; //choose probability density function of Higgs mass arround the measured central value (HiggsSignals)
  setup_pdf_(pdf);
#endif
#endif
}

bool xSM2::UserAnalysis(PhiRef & Phi,LambdaRef & L,MassRef & Mass, MmixingRef & Mixing){
  //////////////////////////////////////////////////////
  // ENTER CODE FOR YOUR TESTS/OUTPUT DURING THE SCAN //
  //////////////////////////////////////////////////////

//--------------------------------------------------//

  //Orders the masses in the h[i] array and computes the ordered alphas and Gammas
  compute_alphas();

  //Test Electroweak precision observables
  if(not TestSTU_2sigma_xSM2_mixall(Mass,Mixing))
    return false;


  
#ifdef HBMODE
//Current implementation uses Hdecay to compute branching ratios for HiggsBounds. Alternatively you can set the values of HiggsBoundsAnalysis using your own calculations.
#ifdef SHDECAYMODE
  if(not PrepareHiggsBoundsInput())
    return false;
  HiggsBoundsAnalysis.run(HBresult,chan,obsratio,ncombined);
  if(HBresult==0)
    return false; //Exclude point if excluded from non-observation in the HB search channels ided pvalue on output.

  //compute the ordered gammas
  OrderGammas();
#ifdef HSMODE  
  //Call to run Higgs Signals 
  int mode=1; //input
  //output variables
  run_higgssignals_(mode,csqmu,csqmh,csqtot,nobs,Pvalue);
  //if(HBresult==0 || Pvalue<0.0027)
#endif  
#endif
#endif

//----------------------------------------------------//
 
#ifdef MICROMEGASMODE
//if micromegas is on AND we have DM candidate 
  if(DM_candidate){
  
    assignMOvalues();
  
    // Compute dark matter contributions from dark matter particle 1 (lightest)
    double Xf,cut=0.01,Beps=1e-5;
    
    // First the relic density contribution
    Omega=darkOmega(&Xf,1,Beps);

    if(Omega>0.13 || Omega<0)
      return false;

    // spin_amplitudes: 0: pA0; 1: pAspin; 2: nA0; 3:nAspin
    double spin_amplitudes[4][2];	
    
    
    if(not test_nucleon_amplitudes(Omega,sigmaSI3,Cp.Mass[2],spin_amplitudes,sigmaup)){
      return false;
    }
  }
#endif

  //----------------------------------------------------//

  /*/RGEtools test.... still not completly stable
  
  vector<double> yi(11);
  //y[0] to y[6] correspond to the couplings L[0] to L[6]; y[7]=gamma_higgs; y[8-10] to g, g', g'' and y[11] is yt
  
  //initial status
  yi[0]=Cp.L[0];
  yi[1]=Cp.L[1];
  yi[2]=Cp.L[2];
  yi[3]=Cp.L[3];
  yi[4]=Cp.L[4];
  yi[5]=Cp.L[5];
  yi[6]=Cp.L[6];
  //g_1; g_2; g_3  
  yi[7]=.461;
  yi[8]=.652;
  yi[9]=1.22;
  //yi[10]=Yt=mTop * sqrt(2)/vev (mTop is the running mass at Mz=171.5GeV)
  yi[10]=.986;
    
  //vector< vector<double> > yfvec;
  //vector<double> tevolution;
  //vector< vector<double> > yfvec2;
  //vector<double> tevolution2;
  
  double tmin=0;
  double tmax=40;*/

  //if(!EvolveBySteps(tmin,tmax,tevolution,yi,yfvec,beta_func,NULL,0,1e-10,npoints))
  //  clog<<"ScannerS Warning: inf or nan found evolving RGE. one or more of the functions have assymptotic behavior."<<endl;
  
  //EvolvePlusCheckBySteps(tmin,tmax,tevolution,yi,yfvec,deriv,NULL,check,NULL,0,1e-10,npoints);
  
  //double tf=EvolvePlusCheckBySteps(tmin,tmax,tevolution,yi,yfvec,beta_func,NULL,RGE_Check_Stability,NULL,0,1e-10,npoints);
  
  /*/---------------------------------------------------------//
  std::clog<<"Initial point couplings:"<<endl;
  for(size_t i=0; i!=Cp.L.size();++i)
    std::clog<<Cp.L[i]<<"\t";
  std::clog<<endl<<"Evolution:"<<endl;
  //---------------------------------------------------------/*/
  
  /*vector<double> yf(11);
  
  void* pvoid=&a0tree;
    
  double tf=EvolvePlusCheckDirect(tmin,tmax,yi,yf,beta_func,NULL,RGE_Stop_criteria,pvoid,0,1e-10);*/
  
  /*/---------------------------------------------------------//
  
  std::clog<<"!!!!!! EvolvePlusCheckKeepAll !!!!!!";
  for(size_t j=0;j!=t_evolve.size();++j){
    std::clog<<endl<<"t: "<<t_evolve[j]<<endl;
    for(int i=0;i!=12;++i){
      std::clog<<y_evolve[j][i]<<"\t";
    }
    std::clog<<endl; 
  }
  
  //---------------------------------------------------------/*/
    
  /*/normal print (point after evolution breaks)  
  for(size_t i=0;i!=yf.size();++i){
    extra_data[i]=yf[i];
  }
  extra_data.back()=tf;
   
  

//--------------------------------------------------/*/

  //clog<<"test: "<<InternalTableSusHiPP_SM(13000.02,125.3)<<endl;
  //exit(0);
     
  return true;
}

void xSM2::what2print(){
//////////////////////////////////////////////////////////////////////////
//       CHOSE WHAT YOU WANT TO BE PRINTED TO THE OUTPUT FILE HERE      //
////////////////////////////////////////////////////////////////////////// 
// YOU NEED TO USE setprint() IN UserInitCalcs() TO DEFINE HOW MUCH     //
// DATA YOU WANT TO PRINT PER POINT (AND HOW MANY POINTS TO STORE       //
// BEFORE PRINTING).                                                    //
////////////////////////////////////////////////////////////////////////// 
  
  size_t data=0;
  //----------------------------------------------------//
  //Masses (3)
  for(size_t i=0;i!=3;++i){
    points2print[store_vector_index][data]=Cp.Mass[h[i]];
    ++data;
  }
  //Vevs (+3=6)
  points2print[store_vector_index][data]=Phi[2];
  ++data;
  for(size_t i=4;i!=6;++i){
    points2print[store_vector_index][data]=Phi[i];
    ++data;
  }
  //Couplings (+7=13)
  for(size_t i=0;i!=7;++i){
    points2print[store_vector_index][data]=Cp.L[i];
    ++data;
  }
  //alphas (+3=16)
  for(size_t i=1;i!=4;++i){
    points2print[store_vector_index][data]=alpha[i];
    ++data;
  }
  //Mixing matrix (+9=25)
  for(size_t i=0;i!=3;++i){
  points2print[store_vector_index][data]=Mmixing[h[i]][2];
  ++data;
  points2print[store_vector_index][data]=Mmixing[h[i]][4];
  ++data;
  points2print[store_vector_index][data]=Mmixing[h[i]][5];
  ++data;
  }
#ifdef HBMODE
#ifdef HSMODE
  //Signal probability (+1=26)
  points2print[store_vector_index][data]=Pvalue;
  ++data;
#endif  
  //Branching Ratios & decay width (+3*12=62)
  for(size_t i=0;i!=nHzero;++i){
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjss[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjcc[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjbb[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjtt[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjmumu[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjtautau[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjWW[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjZZ[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjZga[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjgaga[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.BR_hjgg[h[i]];
    ++data;
    points2print[store_vector_index][data]=HiggsBoundsAnalysis.MhGammaTot[h[i]];
    ++data; 
  }
#endif 
  //Scalar widths (+4=66)
  for(size_t i=0;i!=4;++i){
    points2print[store_vector_index][data]=GammaOrdered[i];
    ++data;
  }
  //xsections (+3=69)
  for(size_t i=0;i!=3;++i){
    points2print[store_vector_index][data]=Mmixing[h[i]][2]*Mmixing[h[i]][2]*InternalTableSusHiPP_SM_noEW(13000.,Cp.Mass[h[i]]);
    ++data;
  }
  //HB and HS parameters (+7=76)
  points2print[store_vector_index][data]=csqmu;
  ++data;
  points2print[store_vector_index][data]=csqmh;
  ++data;
  points2print[store_vector_index][data]=csqtot;
  ++data;
  points2print[store_vector_index][data]=nobs;
  ++data;
  points2print[store_vector_index][data]=chan;
  ++data;  
  points2print[store_vector_index][data]=ncombined;
  ++data;
  points2print[store_vector_index][data]=obsratio;
  ++data;
#ifdef MICROMEGASMODE
  points2print[store_vector_index][data]=Omega;
  ++data;
  points2print[store_vector_index][data]=sigmaSI3;
  ++data;
  points2print[store_vector_index][data]=sigmaup;
  ++data;
#endif
  //////////////////////////---------------------//////////////////////*/

  //-----------------------Extra DATA--------------------//

  for(size_t i=0;i!=extra_data.size();++i){
    points2print[store_vector_index][data]=extra_data[i];
    ++data;
  }
} 

void xSM2::UserFinalCalcs(void){
  ////////////////////////////////////////////////////////////////////////
  // ENTER HERE CODE FOR YOUR FINAL CALCULATIONS AFTER THE SCAN IS DONE //
  ////////////////////////////////////////////////////////////////////////
  // Function for User defined final calculations to be executed after	//
  // the scan is finished						//
  ////////////////////////////////////////////////////////////////////////
  
}

bool xSM2::CheckStability(LambdaRef & L)const{
  ///////////////////////////////////////////////////////////////////
  ///// xSM2 Potential Stability function                       /////
  ///////////////////////////////////////////////////////////////////
  ///// THIS FUNCTION IS ON A USER.CPP FILE MAINLY TO SHOW THE	/////
  ///// USER WHAT THE SCAN IS CALCULATING. WE WOULD ADVISE YOU	/////
  ///// NOT TO ALTER THE CONTENTS OF THIS FUNCTION.		/////
  ///////////////////////////////////////////////////////////////////
  if(L[1]<0 || L[4]<0 || (L[2]<0 && pow(L[2],2)>L[1]*L[4])){
#ifdef VERBOSE
    std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckStability(LambdaRef & L)."<<endl;
#endif
    return false;
  }
  return true;
}

bool xSM2::CheckGlobal(PhiRef & Phi0,LambdaRef & L0,Potential & V)const{
  ///////////////////////////////////////////////////////////////////
  ///// xSM2 Global Minimum function                            /////
  ///////////////////////////////////////////////////////////////////
  ///// THIS FUNCTION IS ON A USER.CPP FILE MAINLY TO SHOW THE	/////
  ///// USER WHAT THE SCAN IS CALCULATING. WE WOULD ADVISE YOU	/////
  ///// NOT TO ALTER THE CONTENTS OF THIS FUNCTION.		/////
  ///////////////////////////////////////////////////////////////////
  
  
  //Bind references to auxiliary potential Vaux
  PhiRef Phi(V); 
  LambdaRef L(V);
  //Copy values from current parameter space point
  Phi.copy(Phi0);
  L.copy(L0);
  
  //// Compute potential at the current minimum
  double V0=V();

  double Vtest;
  double * atemp;
  atemp = new double [3+1];
  double * ztemp;
  ztemp = new double [2*3];
  //Possible minimum with H=0 and A=0
  atemp[0]=L[6]*sqrt(2);
  atemp[1]=L[3]/2+L[5]/2;
  atemp[2]=0;
  atemp[3]=L[4]/4;
  polyroots(atemp,ztemp,3);
  for(int i=0;i<3;++i){
    if(abs(ztemp[2*i+1])<1e-8*abs(ztemp[2*i])){
      Phi[2]=0e0;
      Phi[4]=ztemp[2*i];
      Phi[5]=0e0;
      Vtest=V();
      if(Vtest<V0)
	return false;
    }
    Phi.copy(Phi0);   
  }
  //Possible minimum with H=0 and A and S non-zero
  double vx = -L[6]*sqrt(2)/L[5];
  double squarevy= 2/L[4]*( L[5] - L[3] - L[4]*L[6]*L[6]/L[5]/L[5] );
  if(squarevy>0){
    Phi[2]=0;
    Phi[4]=vx;
    Phi[5]=sqrt(squarevy);
    Vtest=V();
    if(Vtest<V0)
      return false;
  }
  Phi.copy(Phi0);

  //Possible minimum with H ans S nonzero and A=0 
  atemp[0]=2*L[6]*sqrt(2);
  atemp[1]=L[3]+L[5]-L[2]*L[0]/L[1];
  atemp[2]=0;
  atemp[3]=L[4]/2e0-L[2]*L[2]/2e0/L[1];
  polyroots(atemp,ztemp,3);
  for(int i=0;i<3;i++){
    if(abs(ztemp[2*i+1])<1e-8*abs(ztemp[2*i])){
      Phi[4]=ztemp[2*i];
      double Xtemp=-(2*L[0]+L[2]*Phi[4]*Phi[4])/L[1];
      if(Xtemp>0){
	Phi[2]=sqrt(Xtemp);
        Phi[5]=0e0;
	Vtest=V();
	if(Vtest<V0)
	  return false;
      }	  
    }
    Phi.copy(Phi0);
  }    

  //Possible minimum with H nonzero and A nonzero
  vx=-L[6]*sqrt(2)/L[5];
  squarevy= 2*(L[5]*L[5]*(L[1]*(L[5]-L[3])+L[0]*L[2])-L[4]*L[6]*L[6]*L[1]+L[2]*L[2]*L[6]*L[6])/(L[4]*L[5]*L[5]*L[1]-L[2]*L[2]*L[5]*L[5]);

  //2*( L[1]*L[5]- L[1]*L[3]+ ( -L[4]*L[1]+L[2]*L[2] )*L[6]*L[6]/L[5/L[5] +L[0]*L[2])/(L[1]*L[4]-L[2]*L[2]);
  double vsquare=-(2*L[0] + L[2] *vx*vx + L[2]*squarevy )/L[1]; 
  if(squarevy>0){
    Phi[2]=sqrt(vsquare);
    Phi[4]=vx;
    Phi[5]=sqrt(squarevy);
    Vtest=V();
    if(Vtest<V0){
#ifdef VERBOSE
    std::clog<<endl<<"---> ScannerS: Point rejected by xSM2::CheckGlobal(PhiRef & Phi0,LambdaRef & L0,Potential & V)."<<endl;
#endif
      return false;
    }
  }
  
  delete [] atemp;
  delete [] ztemp;
  
  return true;
}

bool RGE_Stop_criteria(const std::vector<double> & y,void *pvoid){
//Use this function to define conditions that will stop the RGE's evolution
//temporary function (TODO: build better alternative later)  
    
  //1st we test stability
  if(y[1]<0 || y[4]<0 || (y[2]<0 && pow(y[2],2)>y[1]*y[4]))
    return false;

//------------------------------------------------------//
    
  //tree level unitarity
  std::vector<std::vector<std::vector<double> > > a0tree=*(static_cast<std::vector<std::vector<std::vector<double> > >*>(pvoid));
  double a0treeSum[441];
  size_t dim=21;
  
  for(size_t i=0; i<dim;++i){
    for(size_t j=0;j<dim;++j){
      a0treeSum[dim*i+j]=0;
      for(size_t a=0; a!=7; ++a){	
	a0treeSum[dim*i+j]+=a0tree[i][j][a]*y[a];
      }
    }
  }

  //Compute Eigenvalues (matrix is destroyed!)
  EigenSystemRsymNoSort(dim,a0treeSum,a0treeSum);

  for(size_t i=0;i<dim;++i){
    if(abs(a0treeSum[i])>0.5)
      return false;
  }

//------------------------------------------------------/*/
    
  return true;
}

//////////////////////////////////////////////////////////
//////// xSM2 default parameterizations         //////////
//////////////////////////////////////////////////////////
// These functions will be called when you use the xSM2 //
// class. xSM2.potential does not have parametrizations //
// on by default, so they will not be called.		//
//////////////////////////////////////////////////////////

void xSM2::MyPhiParametrization(const PhiParamVec & PhiPar,PhiVec & Phi){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE xSM2.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////
  
}

void xSM2::MyInternalMixing(const PhiParamVec & PhiPar,const PhiVec & Phi,MixingparamVec & MixPar,vector< vector<double> > & MixInternal){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE xSM2.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////

}

void xSM2::MyCoupMassRelations(const PhiParamVec & PhiPar,const PhiVec & Phi,const vector< vector<double> > & Mixing,const MixingparamVec & MixPar,vector<double> & L,vector<double> & Mass, MLparamVec & MLPar, vector<double> & IndependentMs,vector<double> & IndependentLs){
  ///////////////////////////////////////////////////////////
  /////// WARNING: THIS FUNCTION WILL NOT BE CALLED BY 	/////
  /////// DEFAULT SINCE THE xSM2.potential FILE 	/////
  /////// DOESN'T HAVE THIS PARAMETRIZATION ON		/////
  ///////////////////////////////////////////////////////////

}
