#include "ScannerSUserDefined.h"
#include "ScannerSInterface.h"
#include <iostream>
#include <fstream>

using namespace std;

int main(int argc,char **argv){

  CheckHelpNeeded(argc,argv);
  MakeInputfile(argc,argv,"--model");

  ifstream instream;
  ofstream outstream, logstream,errstream;
  string inputfile, outputfile,logfile,errfile;
  streambuf *outbuff,*logbuff,*errbuff;

  int used_arg_number=1; //stores the number of used arguments in the command line (--help stops the program so there is no nned to check for that)

  used_arg_number+=RedirectOstream(cout,outstream,outbuff,argc,argv,outputfile,"-o");
  used_arg_number+=RedirectInfostream(std::clog,logstream,logbuff,argc,argv,logfile,"--log");
  used_arg_number+=RedirectInfostream(cerr,errstream,errbuff,argc,argv,errfile,"--err");
  used_arg_number+=OpenInputFile(instream,argc,argv,inputfile,"-i");

  string seedstring=ReturnOption(argc,argv,"--seed");
  unsigned int seed = atoi(seedstring.c_str());
  RandGen r1(seed);
  if(seedstring.size()!=0)
    used_arg_number+=2;

  string snscan=ReturnOption(argc,argv,"--nscan");

  unsigned int nscan=1;
  if(snscan.size()!=0){
    nscan=atoi(snscan.c_str());
    used_arg_number+=2;
  }

  if(used_arg_number!=argc){
    cerr<<"ScannerS ERROR: Inconsistent command line arguments. Please check the manual or use the --help command."<<endl;
    exit(EXIT_FAILURE);
  }
  Potential *V=NULL;
  Potential *Vaux=NULL;
  chose_model(instream,V,Vaux);

  instream>>*V;
  instream.close();
  instream.clear();
  *Vaux=*V;  //make a copy of the potential that may be needed for CheckGlobal
  PhiRef Phi(*V);
  LambdaRef L(*V);
  MassRef Mass(*V);
  MmixingRef Mixing(*V);

  unsigned int sizenscan=log10(nscan)+1;

  V->PrintHeader();
  V->UserInitCalcs();

  std::clog<< "Generating points in parameter space... "<< endl;
  for(unsigned int i=0; i!=nscan;){
    if(
       V->generate() // Generate a parameter space point with all Vevs, Mixings, Couplings and Masses consistently determined.
       && V->CheckBounds() //Check that the point generated is in the hypercubic region allowed by the user.
       && V->CheckTreeUni() //Optional check if the combination of quartic couplings that has been chosen is consistent with tree levek unitarity
       && V->CheckStability(L) // Check if the potential is bounded from below (user defined in this version)
       && V->CheckGlobal(Phi,L,*Vaux) // Check for other minimae (user defined in this version)
       && V->UserAnalysis(Phi,L,Mass,Mixing) // User defined tests and analysis if all previous conditions are fulfilled
       ){
      V->print_or_store(std::cout);
#ifndef VERBOSE
      std::clog << string(18+sizenscan+int(log10(i+1e-10)),'\b');
#endif
      ++i;
      std::clog <<  i << " of "<<nscan<<" points done!"<< flush;
    }
  }
  V->print_remaining_points(std::cout);

  std::clog << endl;

  V->UserFinalCalcs();
}
