#include "ScannerSTwoRxSM.h"

using namespace std;

string TwoRxSM::filepath="ScannerScore/ScannerSModels/TwoRxSM/TwoRxSM.potential";
int TwoRxSM::nHzero;
int TwoRxSM::nHplus=0;
int TwoRxSM::whichanalyses=3;	//which Higgs bounds analysis
int TwoRxSM::whichinput=1; 	//which type of input you will provide (1=hadronic)

// CONSTRUCTOR
TwoRxSM::TwoRxSM():Gamma_const(1./32./acos(-1)){

  //heavy particle masses
  for(int i=0;i!=3;++i)
    mass_bounds[i]=false;
  //goldstone masses
  for(int i=3;i!=6;++i)
    mass_bounds[i]=true;
  //couplings
  for(int i=0;i!=12;++i)
    L_bounds[i]=false;
  //VeV's: 0 to 3 are Higgs' (0,1 and 3 being fixed at 0) and 4:5 are the new Singlet's
  phi_bounds[0]=true;
  phi_bounds[1]=true;
  phi_bounds[2]=false;
  phi_bounds[3]=true;
  phi_bounds[4]=false;
  phi_bounds[5]=false;
  //DM bool
  DM_candidate=false;
}

std::string TwoRxSM::name(){
//returns the name of the type of potential used.  
  string name("TwoRxSM");	
  return name;
}

void TwoRxSM::readfile(std::istream & in){
//This function is called from the overloaded operator >> when a TwoRxSM input file is provided.
//On the base class, Potential::readfile simply calls openinput() on "in".
//Since TwoHDM files only contain the relevante variables, "this" readfile opens an auxiliary file in the same format has Potential::readfile
//and then replaces the relevante variables bty the values in "in".
  ifstream modelfile;
  OpenModelFile(modelfile,filepath);
  openinput(modelfile);
  
  //VeVs
  phi_bounds[2]=getvariable(in,Phi.Vecmin[2],Phi.Vecmax[2],">vH");
  phi_bounds[4]=getvariable(in,Phi.Vecmin[4],Phi.Vecmax[4],">vS");
  phi_bounds[5]=getvariable(in,Phi.Vecmin[5],Phi.Vecmax[5],">vA");
  //masses
  mass_bounds[0]=getvariable(in,Cp.Mass.Vecmin[0],Cp.Mass.Vecmax[0],">massH");
  mass_bounds[1]=getvariable(in,Cp.Mass.Vecmin[1],Cp.Mass.Vecmax[1],">massS");
  mass_bounds[2]=getvariable(in,Cp.Mass.Vecmin[2],Cp.Mass.Vecmax[2],">massA");
  //coupling
  L_bounds[0]=getvariable(in,Cp.L.Vecmin[0],Cp.L.Vecmax[0],">m^2");
  L_bounds[1]=getvariable(in,Cp.L.Vecmin[1],Cp.L.Vecmax[1],">mS^2");
  L_bounds[2]=getvariable(in,Cp.L.Vecmin[2],Cp.L.Vecmax[2],">mA^2");
  L_bounds[3]=getvariable(in,Cp.L.Vecmin[3],Cp.L.Vecmax[3],">mu_S");
  L_bounds[4]=getvariable(in,Cp.L.Vecmin[4],Cp.L.Vecmax[4],">mu_SH");
  L_bounds[5]=getvariable(in,Cp.L.Vecmin[5],Cp.L.Vecmax[5],">mu_AS");  
  L_bounds[6]=getvariable(in,Cp.L.Vecmin[6],Cp.L.Vecmax[6],">lambda_SH");
  L_bounds[7]=getvariable(in,Cp.L.Vecmin[7],Cp.L.Vecmax[7],">lambda_AS");
  L_bounds[8]=getvariable(in,Cp.L.Vecmin[8],Cp.L.Vecmax[8],">lambda_AH");
  L_bounds[9]=getvariable(in,Cp.L.Vecmin[9],Cp.L.Vecmax[9],">lambda_A");
  L_bounds[10]=getvariable(in,Cp.L.Vecmin[10],Cp.L.Vecmax[10],">lambda_S");
  L_bounds[11]=getvariable(in,Cp.L.Vecmin[11],Cp.L.Vecmax[11],">lambda");
  
  //always independent
  for(int i=0;i!=6;++i){
    if(!phi_bounds[i]){
      cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the TwoRxSM input file. This model ALWAYS requires you define the range for vH, vS, vA, mH, mS, mA, lambda_SH, lambda_S and lambda_AS (aditionally you need to provide ranges for lambda_A, lambda_AH and mu_AS in the DM case). These are independent parameters that will be randomly generated."<<endl;
      exit(EXIT_FAILURE);
    }
  }
  for(int i=0;i!=6;++i){
    if(!mass_bounds[i]){
      cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the TwoRxSM input file. This model ALWAYS requires you define the range for vH, vS, vA, mH, mS, mA, lambda_SH, lambda_S and lambda_AS (aditionally you need to provide ranges for lambda_A, lambda_AH and mu_AS in the DM case). These are independent parameters that will be randomly generated."<<endl;
      exit(EXIT_FAILURE);
    }
  }
  if((!L_bounds[6]) || (!L_bounds[7]) || (!L_bounds[10])){
    cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the TwoRxSM input file. This model ALWAYS requires you define the range for vH, vS, vA, mH, mS, mA, lambda_SH, lambda_S and lambda_AS (aditionally you need to provide ranges for lambda_A, lambda_AH and mu_AS in the DM case). These are independent parameters that will be randomly generated."<<endl;
    exit(EXIT_FAILURE);
  }
  //DM phase
  if(Phi.Vecmin[5]==0 && Phi.Vecmax[5]==0){
    DM_candidate=true;
    nHzero=2;
    if((!L_bounds[5]) || (!L_bounds[8]) || (!L_bounds[9])){
      cerr<<"ScannerS ERROR: Unable to find a necessary parameter in the TwoRxSM input file. This model ALWAYS requires you define the range for vH, vS, vA, mH, mS, mA, lambda_SH, lambda_S and lambda_AS (aditionally you need to provide ranges for lambda_A, lambda_AH and mu_AS in the DM case). These are independent parameters that will be randomly generated."<<endl;
      exit(EXIT_FAILURE);
    }
  }
  else{	//broken phase
    DM_candidate=false;
    nHzero=3;
  }
#ifdef HBMODE
  HiggsBoundsAnalysis.rewrite_values(nHzero,nHplus,whichanalyses,whichinput); //sets the values needed to initialize HiggsBounds (will be initialized in UserInitCalcs)
#endif
}

#ifdef HDECAYMODE
#ifdef HBMODE
bool TwoRxSM::PrepareHiggsBoundsInput(){
	
  /////////////////////////////////////////////////////////
  //Passing couplings, masses and decay properties to HB //
  /////////////////////////////////////////////////////////

  for(int i=0;i!=nHzero;++i){
    //Call of Hdecay to compute all branching ratios
    CreateInputFileHdecaySM(Cp.Mass[i]);
    HdecayCalcSM(HdecayHsm);
    
    double Mi2sq=Mmixing[i][2]*Mmixing[i][2];
    BRfrac=1/(1+GammaS[i]/(Mi2sq*HdecayHsm.at("Width")));
    
    if(HdecayHsm.at("Width")==0){
      std::clog<<"All SM Hdecay partial Widths are 0 (this is likely to happen for small masses, current mass MH_sm is "<<Cp.Mass[i]<<"). This point will be rejected."<<endl;
      return false;
    }
    
    HiggsBoundsAnalysis.Mh[i]=Cp.Mass[i];
    HiggsBoundsAnalysis.CP[i]=1;
    //Branching ratios and total width
    HiggsBoundsAnalysis.BR_hjss[i] = BRfrac*HdecayHsm.at("BR(Hsm -> s sbar)");
    HiggsBoundsAnalysis.BR_hjcc[i] = BRfrac*HdecayHsm.at("BR(Hsm -> c cbar)");
    HiggsBoundsAnalysis.BR_hjbb[i] = BRfrac*HdecayHsm.at("BR(Hsm -> b bbar)");
    HiggsBoundsAnalysis.BR_hjmumu[i] = BRfrac*HdecayHsm.at("BR(Hsm -> mu+ mu-)");
    HiggsBoundsAnalysis.BR_hjtautau[i] = BRfrac*HdecayHsm.at("BR(Hsm -> tau+ tau-)");
    HiggsBoundsAnalysis.BR_hjWW[i] = BRfrac*HdecayHsm.at("BR(Hsm -> W+ W-)");
    HiggsBoundsAnalysis.BR_hjZZ[i] = BRfrac*HdecayHsm.at("BR(Hsm -> Z Z)");
    HiggsBoundsAnalysis.BR_hjZga[i]= BRfrac*HdecayHsm.at("BR(Hsm -> Z gamma)");
    HiggsBoundsAnalysis.BR_hjgaga[i] = BRfrac*HdecayHsm.at("BR(Hsm -> gamma gamma)");
    HiggsBoundsAnalysis.BR_hjgg[i] = BRfrac*HdecayHsm.at("BR(Hsm -> g g)");
    HiggsBoundsAnalysis.MhGammaTot[i] = Mi2sq*HdecayHsm.at("Width")+GammaS[i];
    //Neutral Higgs hadronic input
    //LEP
    HiggsBoundsAnalysis.CS_lep_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lep_bbhj_ratio[i]=Mi2sq; 
    HiggsBoundsAnalysis.CS_lep_tautauhj_ratio[i]=Mi2sq; 
    //Tevatron
    HiggsBoundsAnalysis.CS_tev_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_tev_vbf_ratio[i]=Mi2sq; 
    HiggsBoundsAnalysis.CS_tev_tthj_ratio[i]=Mi2sq;
    //LHC7
    HiggsBoundsAnalysis.CS_lhc7_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_vbf_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc7_tthj_ratio[i]=Mi2sq;
    //LHC8
    HiggsBoundsAnalysis.CS_lhc8_hj_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjb_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjW_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_hjZ_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_vbf_ratio[i]=Mi2sq;
    HiggsBoundsAnalysis.CS_lhc8_tthj_ratio[i]=Mi2sq;
  }
  
  //branching ratios for higgs to 2 higgs decay
  if(DM_candidate){
    HiggsBoundsAnalysis.BR_hjhihi[0][1] = Gamma011/HiggsBoundsAnalysis.MhGammaTot[0];
    HiggsBoundsAnalysis.BR_hjhihi[1][0] = Gamma100/HiggsBoundsAnalysis.MhGammaTot[1];
    HiggsBoundsAnalysis.BR_hjinvisible[0] = Gamma022/HiggsBoundsAnalysis.MhGammaTot[0];
    HiggsBoundsAnalysis.BR_hjinvisible[1] = Gamma122/HiggsBoundsAnalysis.MhGammaTot[1];
  }
  else{
    HiggsBoundsAnalysis.BR_hjhihi[0][1] = Gamma011/HiggsBoundsAnalysis.MhGammaTot[0];
    HiggsBoundsAnalysis.BR_hjhihi[0][2] = Gamma022/HiggsBoundsAnalysis.MhGammaTot[0];
    HiggsBoundsAnalysis.BR_hjhihi[1][0] = Gamma100/HiggsBoundsAnalysis.MhGammaTot[1];
    HiggsBoundsAnalysis.BR_hjhihi[1][2] = Gamma122/HiggsBoundsAnalysis.MhGammaTot[1];
    HiggsBoundsAnalysis.BR_hjhihi[2][0] = Gamma200/HiggsBoundsAnalysis.MhGammaTot[2];
    HiggsBoundsAnalysis.BR_hjhihi[2][1] = Gamma211/HiggsBoundsAnalysis.MhGammaTot[2];
  }  
  return true;
}
#endif
#endif

void TwoRxSM::PrintHeader() const{
  //NEEDS TO BE ADAPTED
  std::clog<<endl<<"***********************************************"<<endl;
  std::clog<<"*********** "<<scanners_version<<" ************"<<endl;
  std::clog<<"***********************************************" <<endl;
  std::clog<<endl<<"---------- SUMMARY INFO FOR THIS RUN ---------- "<<endl;
  std::clog<<endl<<"Potential Type: TwoRxSM"<<endl;
  std::clog<<endl<<"V(H,S) = "<<endl;
  std::clog<<endl<< "The expanded potential is:"<<endl;
  std::clog<<endl<<(*this)<<endl;
  std::clog<<endl<<"The ranges of parameters to be scanned are:"<<endl;
  std::clog<<endl<<"Ranges for vacuum expectation values:"<<endl;
  for(size_t i=0;i!=Phi.size();++i){
    if(phi_bounds[i])
      std::clog<<"phi"<<i<<" in ["<<Phi.vgetmin(i)<<":"<<Phi.vgetmax(i)<<"]"<< endl;
  }
  std::clog <<endl<<"Ranges for couplings:"<< endl;
  for(size_t i=0;i!=Cp.L.size();++i){
    if(L_bounds[i])
      std::clog<<"L"<<i<<" in ["<<Cp.L.vgetmin(i)<<":"<<Cp.L.vgetmax(i)<<"]"<< endl;
  }
  std::clog <<endl<<"Ranges for physical state masses:"<< endl;
  for(size_t i=0;i!=Cp.Mass.size();++i){
    if(mass_bounds[i])
    std::clog<<"m"<<i<<" in ["<<Cp.Mass.vgetmin(i)<<":"<<Cp.Mass.vgetmax(i)<<"]"<<endl;
  }
  std::clog<<endl;
  std::clog<<"The seed you are using for the random number generator is: "<<r.getseed()<< endl<< endl;

}

bool TwoRxSM::CheckBounds(){
//unlike Potential::CheckBounds(), this only checks dependent parameters for which the range was explicity declared in the TwoRxSM input file. if they were not declared in input file (and were loaded of TwoRxSM.potential by default) mass_bounds and L_bounds will be false and the parameters will not be checked
  for(int i=0;i!=6;++i)
    if(mass_bounds[i] && (Cp.Mass[i]<Cp.Mass.Vecmin[i] || Cp.Mass[i]>Cp.Mass.Vecmax[i]))
      return false;
  for(int i=0;i!=6;++i)
    if(phi_bounds[i] && (Phi[i]<Phi.Vecmin[i] || Phi[i]>Phi.Vecmax[i]))
      return false;
  for(int i=0;i!=12;++i)
    if(L_bounds[i] && (Cp.L[i]<Cp.L.Vecmin[i] || Cp.L[i]>Cp.L.Vecmax[i]))
      return false;
      
  return true;
}

double TwoRxSM::compute_scalar_widths(){
//Computes the scallar widths for mix_all 
  if(DM_candidate){
    g011 = ((3*Phi[2]*Cp.L[11]*Mmixing[0][2]*pow(-Mmixing[1][2],2))/2. + (Cp.L[4]*pow(-Mmixing[1][2],2)*Mmixing[1][2])/2. + (Phi[4]*Cp.L[6]*pow(-Mmixing[1][2],2)*Mmixing[1][2])/2. + Cp.L[4]*Mmixing[0][2]*-Mmixing[1][2]*Mmixing[0][2] + Phi[2]*Cp.L[6]*-Mmixing[1][2]*Mmixing[1][2]*Mmixing[0][2] + Cp.L[3]*Mmixing[1][2]*pow(Mmixing[0][2],2) + Phi[4]*Cp.L[10]*Mmixing[1][2]*pow(Mmixing[0][2],2) + (Cp.L[6]*Mmixing[0][2]*(4*Phi[4]*-Mmixing[1][2]*Mmixing[0][2] + 2*Phi[2]*pow(Mmixing[0][2],2)))/4.)/2.;

    g100 = ((3*Phi[2]*Cp.L[11]*pow(Mmixing[0][2],2)*-Mmixing[1][2])/2. + Cp.L[3]*pow(Mmixing[1][2],2)*Mmixing[0][2] + Phi[4]*Cp.L[10]*pow(Mmixing[1][2],2)*Mmixing[0][2] + (Cp.L[4]*(4*Mmixing[0][2]*-Mmixing[1][2]*Mmixing[1][2] + 2*pow(Mmixing[0][2],2)*Mmixing[0][2]))/4. + (Cp.L[6]*(8*Phi[4]*Mmixing[0][2]*-Mmixing[1][2]*Mmixing[1][2] + 4*Phi[2]*-Mmixing[1][2]*pow(Mmixing[1][2],2) + 4*Phi[4]*pow(Mmixing[0][2],2)*Mmixing[0][2] + 8*Phi[2]*Mmixing[0][2]*Mmixing[1][2]*Mmixing[0][2]))/8.)/2.;

    g022 = ((Phi[2]*Cp.L[8]*Mmixing[0][2])/2. + Cp.L[5]*Mmixing[1][2] + Phi[4]*Cp.L[7]*Mmixing[1][2])/2.;

    g012 = 0;

    g122 = ((Phi[2]*Cp.L[8]*-Mmixing[1][2])/2. + Cp.L[5]*Mmixing[0][2] + Phi[4]*Cp.L[7]*Mmixing[0][2])/2.;

    g200 = 0;

    g211 = 0;
  }
  else{
    g011 = ((3*Phi[2]*Cp.L[11]*Mmixing[0][2]*pow(Mmixing[0][4],2))/2. + (Cp.L[4]*pow(Mmixing[0][4],2)*Mmixing[1][2])/2. + (Phi[4]*Cp.L[6]*pow(Mmixing[0][4],2)*Mmixing[1][2])/2. + Cp.L[4]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[1][4] + Phi[2]*Cp.L[6]*Mmixing[0][4]*Mmixing[1][2]*Mmixing[1][4] + Cp.L[3]*Mmixing[1][2]*pow(Mmixing[1][4],2) + Phi[4]*Cp.L[10]*Mmixing[1][2]*pow(Mmixing[1][4],2) + (Cp.L[6]*Mmixing[0][2]*(4*Phi[4]*Mmixing[0][4]*Mmixing[1][4] + 2*Phi[2]*pow(Mmixing[1][4],2)))/4. + (Phi[5]*Cp.L[8]*pow(Mmixing[0][4],2)*Mmixing[2][2])/2. + Phi[5]*Cp.L[7]*pow(Mmixing[1][4],2)*Mmixing[2][2] + Phi[2]*Cp.L[8]*Mmixing[0][4]*Mmixing[2][2]*Mmixing[2][4] + 2*Cp.L[5]*Mmixing[1][4]*Mmixing[2][2]*Mmixing[2][4] + 2*Phi[4]*Cp.L[7]*Mmixing[1][4]*Mmixing[2][2]*Mmixing[2][4] + Cp.L[5]*Mmixing[1][2]*pow(Mmixing[2][4],2) + Phi[5]*Cp.L[9]*Mmixing[2][2]*pow(Mmixing[2][4],2) + (Cp.L[8]*Mmixing[0][2]*(4*Phi[5]*Mmixing[0][4]*Mmixing[2][4] + 2*Phi[2]*pow(Mmixing[2][4],2)))/4. + (Cp.L[7]*Mmixing[1][2]*(4*Phi[5]*Mmixing[1][4]*Mmixing[2][4] + 2*Phi[4]*pow(Mmixing[2][4],2)))/2.)/2.;

    g100 = ((3*Phi[2]*Cp.L[11]*pow(Mmixing[0][2],2)*Mmixing[0][4])/2. + Cp.L[3]*pow(Mmixing[1][2],2)*Mmixing[1][4] + Phi[4]*Cp.L[10]*pow(Mmixing[1][2],2)*Mmixing[1][4] + (Cp.L[4]*(4*Mmixing[0][2]*Mmixing[0][4]*Mmixing[1][2] + 2*pow(Mmixing[0][2],2)*Mmixing[1][4]))/4. + (Cp.L[6]*(8*Phi[4]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[1][2] + 4*Phi[2]*Mmixing[0][4]*pow(Mmixing[1][2],2) + 4*Phi[4]*pow(Mmixing[0][2],2)*Mmixing[1][4] + 8*Phi[2]*Mmixing[0][2]*Mmixing[1][2]*Mmixing[1][4]))/8. + Phi[5]*Cp.L[9]*pow(Mmixing[2][2],2)*Mmixing[2][4] + (Cp.L[8]*(8*Phi[5]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[2][2] + 4*Phi[2]*Mmixing[0][4]*pow(Mmixing[2][2],2) + 4*Phi[5]*pow(Mmixing[0][2],2)*Mmixing[2][4] + 8*Phi[2]*Mmixing[0][2]*Mmixing[2][2]*Mmixing[2][4]))/8. + (Cp.L[5]*(2*Mmixing[1][4]*pow(Mmixing[2][2],2) + 4*Mmixing[1][2]*Mmixing[2][2]*Mmixing[2][4]))/2. + (Cp.L[7]*(8*Phi[5]*Mmixing[1][2]*Mmixing[1][4]*Mmixing[2][2] + 4*Phi[4]*Mmixing[1][4]*pow(Mmixing[2][2],2) + 4*Phi[5]*pow(Mmixing[1][2],2)*Mmixing[2][4] + 8*Phi[4]*Mmixing[1][2]*Mmixing[2][2]*Mmixing[2][4]))/4.)/2.;

    g022 = ((3*Phi[2]*Cp.L[11]*Mmixing[0][2]*pow(Mmixing[0][5],2))/2. + (Cp.L[4]*pow(Mmixing[0][5],2)*Mmixing[1][2])/2. + (Phi[4]*Cp.L[6]*pow(Mmixing[0][5],2)*Mmixing[1][2])/2. + Cp.L[4]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[1][5] + Phi[2]*Cp.L[6]*Mmixing[0][5]*Mmixing[1][2]*Mmixing[1][5] + Cp.L[3]*Mmixing[1][2]*pow(Mmixing[1][5],2) + Phi[4]*Cp.L[10]*Mmixing[1][2]*pow(Mmixing[1][5],2) + (Cp.L[6]*Mmixing[0][2]*(4*Phi[4]*Mmixing[0][5]*Mmixing[1][5] + 2*Phi[2]*pow(Mmixing[1][5],2)))/4. + (Phi[5]*Cp.L[8]*pow(Mmixing[0][5],2)*Mmixing[2][2])/2. + Phi[5]*Cp.L[7]*pow(Mmixing[1][5],2)*Mmixing[2][2] + Phi[2]*Cp.L[8]*Mmixing[0][5]*Mmixing[2][2]*Mmixing[2][5] + 2*Cp.L[5]*Mmixing[1][5]*Mmixing[2][2]*Mmixing[2][5] + 2*Phi[4]*Cp.L[7]*Mmixing[1][5]*Mmixing[2][2]*Mmixing[2][5] + Cp.L[5]*Mmixing[1][2]*pow(Mmixing[2][5],2) + Phi[5]*Cp.L[9]*Mmixing[2][2]*pow(Mmixing[2][5],2) + (Cp.L[8]*Mmixing[0][2]*(4*Phi[5]*Mmixing[0][5]*Mmixing[2][5] + 2*Phi[2]*pow(Mmixing[2][5],2)))/4. + (Cp.L[7]*Mmixing[1][2]*(4*Phi[5]*Mmixing[1][5]*Mmixing[2][5] + 2*Phi[4]*pow(Mmixing[2][5],2)))/2.)/2.;

    g012 = (3*Phi[2]*Cp.L[11]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[0][5])/2. + (Cp.L[4]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[1][2])/2. + (Phi[4]*Cp.L[6]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[1][2])/2. + (Cp.L[4]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[1][4])/2. + (Phi[4]*Cp.L[6]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[1][4])/2. + (Phi[2]*Cp.L[6]*Mmixing[0][5]*Mmixing[1][2]*Mmixing[1][4])/2. + (Cp.L[4]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[1][5])/2. + (Phi[4]*Cp.L[6]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[1][5])/2. + (Phi[2]*Cp.L[6]*Mmixing[0][4]*Mmixing[1][2]*Mmixing[1][5])/2. + (Phi[2]*Cp.L[6]*Mmixing[0][2]*Mmixing[1][4]*Mmixing[1][5])/2. + Cp.L[3]*Mmixing[1][2]*Mmixing[1][4]*Mmixing[1][5] + Phi[4]*Cp.L[10]*Mmixing[1][2]*Mmixing[1][4]*Mmixing[1][5] + (Phi[5]*Cp.L[8]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[2][2])/2. + Phi[5]*Cp.L[7]*Mmixing[1][4]*Mmixing[1][5]*Mmixing[2][2] + (Phi[5]*Cp.L[8]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[2][4])/2. + Phi[5]*Cp.L[7]*Mmixing[1][2]*Mmixing[1][5]*Mmixing[2][4] + (Phi[2]*Cp.L[8]*Mmixing[0][5]*Mmixing[2][2]*Mmixing[2][4])/2. + Cp.L[5]*Mmixing[1][5]*Mmixing[2][2]*Mmixing[2][4] + Phi[4]*Cp.L[7]*Mmixing[1][5]*Mmixing[2][2]*Mmixing[2][4] + (Phi[5]*Cp.L[8]*Mmixing[0][2]*Mmixing[0][4]*Mmixing[2][5])/2. + Phi[5]*Cp.L[7]*Mmixing[1][2]*Mmixing[1][4]*Mmixing[2][5] + (Phi[2]*Cp.L[8]*Mmixing[0][4]*Mmixing[2][2]*Mmixing[2][5])/2. + Cp.L[5]*Mmixing[1][4]*Mmixing[2][2]*Mmixing[2][5] + Phi[4]*Cp.L[7]*Mmixing[1][4]*Mmixing[2][2]*Mmixing[2][5] + (Phi[2]*Cp.L[8]*Mmixing[0][2]*Mmixing[2][4]*Mmixing[2][5])/2. + Cp.L[5]*Mmixing[1][2]*Mmixing[2][4]*Mmixing[2][5] + Phi[4]*Cp.L[7]*Mmixing[1][2]*Mmixing[2][4]*Mmixing[2][5] + Phi[5]*Cp.L[9]*Mmixing[2][2]*Mmixing[2][4]*Mmixing[2][5];

    g122 = ((3*Phi[2]*Cp.L[11]*Mmixing[0][4]*pow(Mmixing[0][5],2))/2. + (Cp.L[4]*pow(Mmixing[0][5],2)*Mmixing[1][4])/2. + (Phi[4]*Cp.L[6]*pow(Mmixing[0][5],2)*Mmixing[1][4])/2. + Cp.L[4]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[1][5] + Phi[2]*Cp.L[6]*Mmixing[0][5]*Mmixing[1][4]*Mmixing[1][5] + Cp.L[3]*Mmixing[1][4]*pow(Mmixing[1][5],2) + Phi[4]*Cp.L[10]*Mmixing[1][4]*pow(Mmixing[1][5],2) + (Cp.L[6]*Mmixing[0][4]*(4*Phi[4]*Mmixing[0][5]*Mmixing[1][5] + 2*Phi[2]*pow(Mmixing[1][5],2)))/4. + (Phi[5]*Cp.L[8]*pow(Mmixing[0][5],2)*Mmixing[2][4])/2. + Phi[5]*Cp.L[7]*pow(Mmixing[1][5],2)*Mmixing[2][4] + Phi[2]*Cp.L[8]*Mmixing[0][5]*Mmixing[2][4]*Mmixing[2][5] + 2*Cp.L[5]*Mmixing[1][5]*Mmixing[2][4]*Mmixing[2][5] + 2*Phi[4]*Cp.L[7]*Mmixing[1][5]*Mmixing[2][4]*Mmixing[2][5] + Cp.L[5]*Mmixing[1][4]*pow(Mmixing[2][5],2) + Phi[5]*Cp.L[9]*Mmixing[2][4]*pow(Mmixing[2][5],2) + (Cp.L[8]*Mmixing[0][4]*(4*Phi[5]*Mmixing[0][5]*Mmixing[2][5] + 2*Phi[2]*pow(Mmixing[2][5],2)))/4. + (Cp.L[7]*Mmixing[1][4]*(4*Phi[5]*Mmixing[1][5]*Mmixing[2][5] + 2*Phi[4]*pow(Mmixing[2][5],2)))/2.)/2.;

    g200 = ((3*Phi[2]*Cp.L[11]*pow(Mmixing[0][2],2)*Mmixing[0][5])/2. + Cp.L[3]*pow(Mmixing[1][2],2)*Mmixing[1][5] + Phi[4]*Cp.L[10]*pow(Mmixing[1][2],2)*Mmixing[1][5] + (Cp.L[4]*(4*Mmixing[0][2]*Mmixing[0][5]*Mmixing[1][2] + 2*pow(Mmixing[0][2],2)*Mmixing[1][5]))/4. + (Cp.L[6]*(8*Phi[4]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[1][2] + 4*Phi[2]*Mmixing[0][5]*pow(Mmixing[1][2],2) + 4*Phi[4]*pow(Mmixing[0][2],2)*Mmixing[1][5] + 8*Phi[2]*Mmixing[0][2]*Mmixing[1][2]*Mmixing[1][5]))/8. + Phi[5]*Cp.L[9]*pow(Mmixing[2][2],2)*Mmixing[2][5] + (Cp.L[8]*(8*Phi[5]*Mmixing[0][2]*Mmixing[0][5]*Mmixing[2][2] + 4*Phi[2]*Mmixing[0][5]*pow(Mmixing[2][2],2) + 4*Phi[5]*pow(Mmixing[0][2],2)*Mmixing[2][5] + 8*Phi[2]*Mmixing[0][2]*Mmixing[2][2]*Mmixing[2][5]))/8. + (Cp.L[5]*(2*Mmixing[1][5]*pow(Mmixing[2][2],2) + 4*Mmixing[1][2]*Mmixing[2][2]*Mmixing[2][5]))/2. + (Cp.L[7]*(8*Phi[5]*Mmixing[1][2]*Mmixing[1][5]*Mmixing[2][2] + 4*Phi[4]*Mmixing[1][5]*pow(Mmixing[2][2],2) + 4*Phi[5]*pow(Mmixing[1][2],2)*Mmixing[2][5] + 8*Phi[4]*Mmixing[1][2]*Mmixing[2][2]*Mmixing[2][5]))/4.)/2.;

    g211 = ((3*Phi[2]*Cp.L[11]*pow(Mmixing[0][4],2)*Mmixing[0][5])/2. + Cp.L[3]*pow(Mmixing[1][4],2)*Mmixing[1][5] + Phi[4]*Cp.L[10]*pow(Mmixing[1][4],2)*Mmixing[1][5] + (Cp.L[4]*(4*Mmixing[0][4]*Mmixing[0][5]*Mmixing[1][4] + 2*pow(Mmixing[0][4],2)*Mmixing[1][5]))/4. + (Cp.L[6]*(8*Phi[4]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[1][4] + 4*Phi[2]*Mmixing[0][5]*pow(Mmixing[1][4],2) + 4*Phi[4]*pow(Mmixing[0][4],2)*Mmixing[1][5] + 8*Phi[2]*Mmixing[0][4]*Mmixing[1][4]*Mmixing[1][5]))/8. + Phi[5]*Cp.L[9]*pow(Mmixing[2][4],2)*Mmixing[2][5] + (Cp.L[8]*(8*Phi[5]*Mmixing[0][4]*Mmixing[0][5]*Mmixing[2][4] + 4*Phi[2]*Mmixing[0][5]*pow(Mmixing[2][4],2) + 4*Phi[5]*pow(Mmixing[0][4],2)*Mmixing[2][5] + 8*Phi[2]*Mmixing[0][4]*Mmixing[2][4]*Mmixing[2][5]))/8. + (Cp.L[5]*(2*Mmixing[1][5]*pow(Mmixing[2][4],2) + 4*Mmixing[1][4]*Mmixing[2][4]*Mmixing[2][5]))/2. + (Cp.L[7]*(8*Phi[5]*Mmixing[1][4]*Mmixing[1][5]*Mmixing[2][4] + 4*Phi[4]*Mmixing[1][5]*pow(Mmixing[2][4],2) + 4*Phi[5]*pow(Mmixing[1][4],2)*Mmixing[2][5] + 8*Phi[4]*Mmixing[1][4]*Mmixing[2][4]*Mmixing[2][5]))/4.)/2.;
  }
  
  //Compute Gamma functions
  double Gfrac1,Gfrac2;
  
  Gfrac1=Cp.Mass[1]*Cp.Mass[1]/Cp.Mass[0]/Cp.Mass[0];	//Gamma 011 & 100
  if(Gfrac1<.25){
    Gamma011=Gamma_const*g011*g011/Cp.Mass[0]*sqrt(1-4*Gfrac1);
    Gamma100=0;
  }
  else if(Gfrac1>4){
    Gamma100=Gamma_const*g100*g100/Cp.Mass[1]*sqrt(1-4./Gfrac1);
    Gamma011=0;
  }
  else{
    Gamma011=0;
    Gamma100=0;
  }
  Gfrac1=Cp.Mass[2]*Cp.Mass[2]/Cp.Mass[0]/Cp.Mass[0];	//Gamma 022 & 200
  if(Gfrac1<.25){
    Gamma022=Gamma_const*g022*g022/Cp.Mass[0]*sqrt(1-4*Gfrac1);
    Gamma200=0;
  }
  else if(Gfrac1>4){
    Gamma200=Gamma_const*g200*g200/Cp.Mass[2]*sqrt(1-4./Gfrac1);
    Gamma022=0;
  }
  else{
    Gamma022=0;
    Gamma200=0;
  }
  Gfrac1=Cp.Mass[2]*Cp.Mass[2]/Cp.Mass[1]/Cp.Mass[1];	//Gamma 122 & 211
  if(Gfrac1<.25){
    Gamma122=Gamma_const*g122*g122/Cp.Mass[1]*sqrt(1-4*Gfrac1);
    Gamma211=0;
  }
  else if(Gfrac1>4){
    Gamma211=Gamma_const*g211*g211/Cp.Mass[2]*sqrt(1-4./Gfrac1);
    Gamma122=0;
  }
  else{
    Gamma122=0;
    Gamma211=0;
  }
  //compute crossed decays
  Gfrac1=pow(Cp.Mass[1]+Cp.Mass[2],2)/Cp.Mass[0]/Cp.Mass[0];	//Gfrac1 is being reused
  Gfrac2=pow(Cp.Mass[1]-Cp.Mass[2],2)/Cp.Mass[0]/Cp.Mass[0];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma012=2*Gamma_const*g012*g012/Cp.Mass[0]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma012=0;
  Gfrac1=pow(Cp.Mass[0]+Cp.Mass[2],2)/Cp.Mass[1]/Cp.Mass[1];
  Gfrac2=pow(Cp.Mass[0]-Cp.Mass[2],2)/Cp.Mass[1]/Cp.Mass[1];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma102=2*Gamma_const*g012*g012/Cp.Mass[1]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma102=0;
  Gfrac1=pow(Cp.Mass[0]+Cp.Mass[1],2)/Cp.Mass[2]/Cp.Mass[2];
  Gfrac2=pow(Cp.Mass[0]-Cp.Mass[1],2)/Cp.Mass[2]/Cp.Mass[2];
  if(Gfrac1<1 && Gfrac2<1)
    Gamma201=2*Gamma_const*g012*g012/Cp.Mass[2]*sqrt(1-Gfrac1)*sqrt(1-Gfrac2);
  else
    Gamma201=0;
  
  GammaS[0]=Gamma011+Gamma022+Gamma012;
  GammaS[1]=Gamma100+Gamma122+Gamma102;
  GammaS[2]=Gamma200+Gamma211+Gamma201;
}

void TwoRxSM::MadGraph_Input(){
   
	ofstream MAD_input("param_card.dat");
	
	MAD_input << "######################################################################"<< endl ; 
  MAD_input <<"## PARAM_CARD AUTOMATICALY GENERATED BY MG5 FOLLOWING UFO MODEL   ####"<< endl ; 
  MAD_input <<"######################################################################"<< endl ; 
  MAD_input <<"##                                                                  ##"<< endl ; 
  MAD_input <<"##  Width set on Auto will be computed following the information    ##"<< endl ; 
  MAD_input <<"##        present in the decay.py files of the model.               ##"<< endl ; 
  MAD_input <<"##        See  arXiv:1402.1178 for more details.                    ##"<< endl ; 
  MAD_input <<"##                                                                  ##"<< endl ; 
  MAD_input <<"######################################################################"<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR CKMBLOCK"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block ckmblock "<< endl ; 
  MAD_input <<"    1 2.277360e-01 # cabi "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR FRBLOCK"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block frblock "<< endl ; 
  MAD_input <<"    1 "<< Phi[4] <<" # vevS "<< endl ; 
  MAD_input <<"    2 "<< Phi[5] <<" # vevA "<< endl ; 
  MAD_input <<"    3 "<< Cp.L[11] <<" # L11 "<< endl ; 
  MAD_input <<"    4 "<< Cp.L[0] <<" # L0 "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR INVSCALAR"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block invscalar "<< endl ; 
  MAD_input <<"    1 "<< Cp.L[6] <<" # L6 "<< endl ; 
  MAD_input <<"    2 "<< Cp.L[8] <<" # L8 "<< endl ; 
  MAD_input <<"    3 "<< Cp.L[10] <<" # L10 "<< endl ; 
  MAD_input <<"    4 "<< Cp.L[9] <<" # L9 "<< endl ; 
  MAD_input <<"    5 "<< Cp.L[7] <<" # L7 "<< endl ; 
  MAD_input <<"    6 "<< Cp.L[4] <<" # L4 "<< endl ; 
  MAD_input <<"    7 "<< Cp.L[3] <<" # L3 "<< endl ; 
  MAD_input <<"    8 "<< Cp.L[5] <<" # L5 "<< endl ; 
  MAD_input <<"    9 "<< Cp.L[1] <<" # L1 "<< endl ; 
  MAD_input <<"   10 "<< Cp.L[2] <<" # L2 "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR MASS"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block mass "<< endl ; 
  MAD_input <<"    1 5.040000e-03 # MD "<< endl ; 
  MAD_input <<"    2 2.550000e-03 # MU "<< endl ; 
  MAD_input <<"    3 1.010000e-01 # MS "<< endl ; 
  MAD_input <<"    4 1.270000e+00 # MC "<< endl ; 
  MAD_input <<"    5 4.700000e+00 # MB "<< endl ; 
  MAD_input <<"    6 1.720000e+02 # MT "<< endl ; 
  MAD_input <<"   11 5.110000e-04 # Me "<< endl ; 
  MAD_input <<"   13 1.056600e-01 # MMU "<< endl ; 
  MAD_input <<"   15 1.777000e+00 # MTA "<< endl ; 
  MAD_input <<"   23 9.118760e+01 # MZ "<< endl ; 
  MAD_input <<"   25 "<< Cp.Mass[0] <<" # MH0 "<< endl ; 
  MAD_input <<"   36 "<< Cp.Mass[1] <<" # MH1 "<< endl ; 
  MAD_input <<"   37 "<< Cp.Mass[2] <<" # MH2 "<< endl ; 
  MAD_input <<"## Dependent parameters, given by model restrictions."<< endl ; 
  MAD_input <<"## Those values should be edited following the "<< endl ; 
  MAD_input <<"## analytical expression. MG5 ignores those values "<< endl ; 
  MAD_input <<"## but they are important for interfacing the output of MG5"<< endl ; 
  MAD_input <<"## to external program such as Pythia."<< endl ; 
  MAD_input <<"  12 0.000000 # ve : 0.0 "<< endl ; 
  MAD_input <<"  14 0.000000 # vm : 0.0 "<< endl ; 
  MAD_input <<"  16 0.000000 # vt : 0.0 "<< endl ; 
  MAD_input <<"  21 0.000000 # g : 0.0 "<< endl ; 
  MAD_input <<"  22 0.000000 # a : 0.0 "<< endl ; 
  MAD_input <<"  24 79.824360 # w+ : cmath.sqrt(MZ__exp__2/2. + cmath.sqrt(MZ__exp__4/4. - (aEW*cmath.pi*MZ__exp__2)/(Gf*sqrt__2))) "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR MIXSCALAR"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block mixscalar "<< endl ; 
  MAD_input <<"    1 "<< Mmixing[0][2] <<" # M02 "<< endl ; 
  MAD_input <<"    2 "<< Mmixing[0][4] <<" # M04 "<< endl ; 
  MAD_input <<"    3 "<< Mmixing[0][5] <<" # M05 "<< endl ; 
  MAD_input <<"    4 "<< Mmixing[1][2] <<" # M12 "<< endl ; 
  MAD_input <<"    5 "<< Mmixing[1][4] <<" # M14 "<< endl ; 
  MAD_input <<"    6 "<< Mmixing[1][5] <<" # M15 "<< endl ; 
  MAD_input <<"    7 "<< Mmixing[2][2] <<" # M22 "<< endl ; 
  MAD_input <<"    8 "<< Mmixing[2][4] <<" # M24 "<< endl ; 
  MAD_input <<"    9 "<< Mmixing[2][5] <<" # M25 "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR SMINPUTS"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block sminputs "<< endl ; 
  MAD_input <<"    1 1.279000e+02 # aEWM1 "<< endl ; 
  MAD_input <<"    2 1.166370e-05 # Gf "<< endl ; 
  MAD_input <<"    3 1.184000e-01 # aS "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR YUKAWA"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"Block yukawa "<< endl ; 
  MAD_input <<"    1 5.040000e-03 # ymdo "<< endl ; 
  MAD_input <<"    2 2.550000e-03 # ymup "<< endl ; 
  MAD_input <<"    3 1.010000e-01 # yms "<< endl ; 
  MAD_input <<"    4 1.270000e+00 # ymc "<< endl ; 
  MAD_input <<"    5 4.700000e+00 # ymb "<< endl ; 
  MAD_input <<"    6 1.720000e+02 # ymt "<< endl ; 
  MAD_input <<"   11 5.110000e-04 # yme "<< endl ; 
  MAD_input <<"   13 1.056600e-01 # ymm "<< endl ; 
  MAD_input <<"   15 1.777000e+00 # ymtau "<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"## INFORMATION FOR DECAY"<< endl ; 
  MAD_input <<"###################################"<< endl ; 
  MAD_input <<"DECAY   6 1.508336e+00 # WT "<< endl ; 
  MAD_input <<"DECAY  23 2.495200e+00 # WZ "<< endl ; 
  MAD_input <<"DECAY  24 2.085000e+00 # WW "<< endl ; 
  MAD_input <<"DECAY  25 4.070000e-03 # WH0 "<< endl ; 
  MAD_input <<"DECAY  36 4.070000e-03 # WH1 "<< endl ; 
  MAD_input <<"DECAY  37 4.070000e-03 # WH2 "<< endl ; 
  MAD_input <<"## Dependent parameters, given by model restrictions."<< endl ; 
  MAD_input <<"## Those values should be edited following the "<< endl ; 
  MAD_input <<"## analytical expression. MG5 ignores those values "<< endl ; 
  MAD_input <<"## but they are important for interfacing the output of MG5"<< endl ; 
  MAD_input <<"## to external program such as Pythia."<< endl ; 
  MAD_input <<"DECAY  1 0.000000 # d : 0.0 "<< endl ; 
  MAD_input <<"DECAY  2 0.000000 # u : 0.0 "<< endl ; 
  MAD_input <<"DECAY  3 0.000000 # s : 0.0 "<< endl ; 
  MAD_input <<"DECAY  4 0.000000 # c : 0.0 "<< endl ; 
  MAD_input <<"DECAY  5 0.000000 # b : 0.0 "<< endl ; 
  MAD_input <<"DECAY  11 0.000000 # e- : 0.0 "<< endl ; 
  MAD_input <<"DECAY  12 0.000000 # ve : 0.0 "<< endl ; 
  MAD_input <<"DECAY  13 0.000000 # mu- : 0.0 "<< endl ; 
  MAD_input <<"DECAY  14 0.000000 # vm : 0.0 "<< endl ; 
  MAD_input <<"DECAY  15 0.000000 # ta- : 0.0 "<< endl ; 
  MAD_input <<"DECAY  16 0.000000 # vt : 0.0 "<< endl ; 
  MAD_input <<"DECAY  21 0.000000 # g : 0.0 "<< endl ; 
  MAD_input <<"DECAY  22 0.000000 # a : 0.0 "<< endl ; 
  MAD_input <<"#==========================================================="<< endl ; 
  MAD_input <<"# QUANTUM NUMBERS OF NEW STATE(S) (NON SM PDG CODE)"<< endl ; 
  MAD_input <<"#==========================================================="<< endl ; 
  MAD_input <<""<< endl ; 
  MAD_input <<"Block QNUMBERS 36  # h3 "<< endl ; 
  MAD_input <<"        1 0  # 3 times electric charge"<< endl ; 
  MAD_input <<"        2 1  # number of spin states (2S+1)"<< endl ; 
  MAD_input <<"        3 1  # colour rep (1: singlet, 3: triplet, 8: octet)"<< endl ; 
  MAD_input <<"        4 0  # Particle/Antiparticle distinction (0=own anti)"<< endl ; 
  MAD_input <<"Block QNUMBERS 37  # h+ "<< endl ; 
  MAD_input <<"        1 0  # 3 times electric charge"<< endl ; 
  MAD_input <<"        2 1  # number of spin states (2S+1)"<< endl ; 
  MAD_input <<"        3 1  # colour rep (1: singlet, 3: triplet, 8: octet)"<< endl ; 
  MAD_input <<"        4 0  # Particle/Antiparticle distinction (0=own anti)"<< endl ; 

}

#ifdef MICROMEGASMODE
void TwoRxSM::assignMOvalues(){
  
  ////// Dark matter
  ////// Relic density
  ForceUG=1;  // to Force Unitary Gauge assign 1 
 
  assignVal("LL0",Cp.L[0]);
  assignVal("LL1",Cp.L[1]);
  assignVal("LL2",Cp.L[2]);
  assignVal("LL3",Cp.L[3]);
  assignVal("LL4",Cp.L[4]);
  assignVal("LL5",Cp.L[5]);
  assignVal("LL6",Cp.L[6]);
  assignVal("LL7",Cp.L[7]);
  assignVal("LL8",Cp.L[8]);
  assignVal("LL9",Cp.L[9]);
  assignVal("LL10",Cp.L[10]);
  assignVal("LL11",Cp.L[11]);
  // mixing matrix
  assignVal("Mmix02",Mmixing[0][2]);
  assignVal("Mmix12",Mmixing[1][2]);
  assignVal("Mmix04",Mmixing[0][4]);
  assignVal("Mmix14",Mmixing[1][4]);
  // VEV
  assignVal("vs",Phi[4]);
  
  assignVal("MH0",Cp.Mass[0]);		
  assignVal("MH1",Cp.Mass[1]);		
  assignVal("MAs",Cp.Mass[2]);  
  char cdmName[10];
  int err=sortOddParticles(cdmName);
  if(err) { cerr<<"Can't calculate "<<cdmName<< endl; exit(-1);}
}
#endif

