#include "ScannerSTools.h"

using namespace std;

double non_valid[4]={sqrt(-1),-sqrt(-1),1./0.,-1./0.};

/********** static data **********/
bool RandGen::seeded=false;
unsigned RandGen::seed=0;
gsl_rng * RandGen::r=NULL;
/*********************************/

//RandGen constructors
RandGen::RandGen():T(gsl_rng_ranlxs0){
  if(!seeded){
    r=gsl_rng_alloc(T);
    gsl_rng_set(r,seed);
    cerr<<"ScannerS ERROR: There was a call to the random number generator before it was seeded."<<endl;
    std::clog<<"Make sure no random number calls are made before seeding."<<endl;
    exit(EXIT_FAILURE);
  }
}

RandGen::RandGen(unsigned int seedin):T(gsl_rng_ranlxs0){
  if(!seeded){
    seed=seedin;
    r=gsl_rng_alloc(T);
    gsl_rng_set(r,seed);
    seeded=true;
  }
  else{
    std::clog<<"ScannnerS Warning: There was an attempt to seed the random number generator after it was already running."<<endl;
    std::clog<<"This attempt will be ignored, The generator will continue to return the sequence initiated by the previous seed."<<endl;
  }
}


//Overloaded output operator for the random number generator
ostream & operator<<(ostream & os,RandGen & rgen){
  os << rgen.seed <<"\t" << rgen.T << "\t"<< rgen.r;
  return os;
}

//Overloaded input operator for the VecTriplet class 
istream & operator >>(istream & in, VecTriplet & T){
  unsigned N;
  double min,max;
  in >> N; // read the number of fields
  if(in){//If read is good...
    for(size_t i=0;i!=N;++i){
      in >>min>>max; // read next two lines with phimin[i], phimax[i]
      if(in){//if read good, push back into vectors
	T.Vecmin.push_back(min);
	T.Vecmax.push_back(max);
	T.Vec.push_back((min+max)/2.); //Create vector at the same time, initialised to middle point
      }
      else
	break;
    }
  }
  else{
    cerr<< "ERROR in istream & operator >>(istream & in, VecTriplet & T):  Bad read of VecTriplet. Check Dimensions in mathematica file." << endl;
    exit(-1); // In case of error crash program
  }
  return in;
} 

//Overloaded output operator for class VecTriplet
ostream & operator <<(ostream & os, const VecTriplet & T){
  os<< T.Vecmin.size() << "\t"; // outupt number of fields
  for(VecTriplet::const_iterator iter=T.Vec.begin(),itermin=T.Vecmin.begin(),itermax=T.Vecmax.begin();itermin!=T.Vecmin.end();)
    os << *itermin++ << "\t" << *itermax++<<"\t" << *iter++ << "\t"; // output phimin and phimax ranges one by one
  return os;
} 

//Overloaded output operator for class Monomial
ostream & operator <<(ostream & os, const Monomial & M){
  os<< M.coeff << " "; // outupt coefficient
  Monomial::const_iterator beg=M.pows.begin(),end=M.pows.end();
  for(Monomial::const_iterator iter=beg;iter!=end;++iter)
    if(*iter!=0)
      os <<"phi"<<iter-beg <<"^"<<*iter << " "; // output phimin and phimax ranges one by one
  return os;
} 

//Overloaded input operator for the Monomial class
istream & operator >>(istream & in, Monomial & M){
  in >> M.coeff; // read the coefficient
  if(in){//If read is good...
    for(Monomial::iterator iter=M.pows.begin();iter!=M.pows.end();){
      in >>*(iter++); // read monomial
      if(not in){
	cerr<< "ERROR in istream & operator >>(istream & in, Monomial & M): Bad read of Monomial. Check dimensions in mathematica file." << endl; 	
	exit(EXIT_FAILURE); // In case of error crash program
      }
    }
  }
  else{
    cerr<< "ERROR in istream & operator >>(istream & in, Monomial & M): Bad read of Monomial. Check Dimensions in mathematica file." << endl; 
    exit(EXIT_FAILURE); // In case of error crash program
  }
  return in;
} 

void polyroots(double * & a,double * & z, int dim){
  
  gsl_poly_complex_workspace * w = gsl_poly_complex_workspace_alloc(dim+1);
  
  gsl_poly_complex_solve (a, dim+1, w, z);
  
  gsl_poly_complex_workspace_free (w);
}

void sort3(int * list){
  
  int temp;
  if(list[0]>list[2]){
    temp=list[2];
    list[2]=list[0];
    list[0]=temp;
  }
  if(list[0]>list[1]){
    temp=list[1];
    list[1]=list[0];
    list[0]=temp;
  }
  else if(list[1]>list[2]){
    temp=list[2];
    list[2]=list[1];
    list[1]=temp;
  }
}

bool is_valid(double num){
  //chechs if the number is non-valid (nan;-nan;inf;-inf)
  bool test;
  double *temp_ptr(&num);
  char* cpoint1=(char*) temp_ptr;
  char* cpoint2;
  
  size_t i=0;
  for(size_t i=0;i!=4;++i){
    test=false;
    temp_ptr=&non_valid[i];
    cpoint2=(char*) temp_ptr;
    for(size_t j=0;j!=sizeof(double);++j){
      if(*(cpoint1+j) != *(cpoint2+j)){
	test=true;
	break;
      }
    }
    if(!test)
      break;
  }
    
  return test;
}
